@@ -49,11 +49,20 @@ except ImportError:
     from ansible.parsing.dataloader import DataLoader
     from ansible.template import Templar
     from ansible.parsing.mod_args import ModuleArgsParser
+    from ansible.parsing.yaml.constructor import AnsibleConstructor
+    from ansible.parsing.yaml.loader import AnsibleLoader
     from ansible.errors import AnsibleParserError
     ANSIBLE_VERSION = 2
 
+    # ansible-lint doesn't need/want to know about encrypted secrets, but it needs
+    # Ansible 2.3+ allows encrypted secrets within yaml files, so we pass a string
+    # as the password to enable such yaml files to be opened and parsed successfully.
+    DEFAULT_VAULT_PASSWORD = 'x'
+
     def parse_yaml_from_file(filepath):
         dl = DataLoader()
+        if hasattr(dl, 'set_vault_password'):
+            dl.set_vault_password(DEFAULT_VAULT_PASSWORD)
         return dl.load_from_file(filepath)
 
     def path_dwim(basedir, given):
@@ -66,6 +75,7 @@ except ImportError:
         dl.set_basedir(basedir)
         templar = Templar(dl, variables=templatevars)
         return templar.template(varname, **kwargs)
+
     try:
         from ansible.plugins import module_loader
     except ImportError:
@@ -76,8 +86,8 @@ FILENAME_KEY = '__file__'
 
 VALID_KEYS = [
     'name', 'action', 'when', 'async', 'poll', 'notify',
-    'first_available_file', 'include', 'include_tasks', 'import_playbook',
-    'import_tasks', 'tags', 'register', 'ignore_errors', 'delegate_to',
+    'first_available_file', 'include', 'import_playbook',
+    'tags', 'register', 'ignore_errors', 'delegate_to',
     'local_action', 'transport', 'remote_user', 'sudo',
     'sudo_user', 'sudo_pass', 'when', 'connection', 'environment', 'args', 'always_run',
     'any_errors_fatal', 'changed_when', 'failed_when', 'check_mode', 'delay',
@@ -193,9 +203,7 @@ def play_children(basedir, item, parent_type, playbook_dir):
         'post_tasks': _taskshandlers_children,
         'block': _taskshandlers_children,
         'include': _include_children,
-        'include_tasks': _include_children,
         'import_playbook': _include_children,
-        'import_tasks': _include_children,
         'roles': _roles_children,
         'dependencies': _roles_children,
         'handlers': _taskshandlers_children,
@@ -235,6 +243,12 @@ def _taskshandlers_children(basedir, k, v, parent_type):
             append_children(th['import_playbook'], basedir, k, parent_type, results)
         elif 'import_tasks' in th:
             append_children(th['import_tasks'], basedir, k, parent_type, results)
+        elif 'import_role' in th:
+            results.extend(_roles_children(basedir, k, [th['import_role'].get('name')], parent_type,
+                                           main=th['import_role'].get('tasks_from', 'main')))
+        elif 'include_role' in th:
+            results.extend(_roles_children(basedir, k, [th['include_role'].get('name')], parent_type,
+                                           main=th['include_role'].get('tasks_from', 'main')))
         elif 'block' in th:
             results.extend(_taskshandlers_children(basedir, k, th['block'], parent_type))
             if 'rescue' in th:
@@ -258,19 +272,20 @@ def append_children(taskhandler, basedir, k, parent_type, results):
     })
 
 
-def _roles_children(basedir, k, v, parent_type):
+def _roles_children(basedir, k, v, parent_type, main='main'):
     results = []
     for role in v:
         if isinstance(role, dict):
             if 'role' in role or 'name' in role:
                 if 'tags' not in role or 'skip_ansible_lint' not in role['tags']:
                     results.extend(_look_for_role_files(basedir,
-                                                        role.get('role', role.get('name'))))
+                                                        role.get('role', role.get('name')),
+                                                        main=main))
             else:
                 raise SystemExit('role dict {0} does not contain a "role" '
                                  'or "name" key'.format(role))
         else:
-            results.extend(_look_for_role_files(basedir, role))
+            results.extend(_look_for_role_files(basedir, role, main=main))
     return results
 
 
@@ -312,7 +327,7 @@ def _rolepath(basedir, role):
     return role_path
 
 
-def _look_for_role_files(basedir, role):
+def _look_for_role_files(basedir, role, main='main'):
     role_path = _rolepath(basedir, role)
     if not role_path:
         return []
@@ -321,7 +336,7 @@ def _look_for_role_files(basedir, role):
 
     for th in ['tasks', 'handlers', 'meta']:
         for ext in ('.yml', '.yaml'):
-            thpath = os.path.join(role_path, th, 'main' + ext)
+            thpath = os.path.join(role_path, th, main + ext)
             if os.path.exists(thpath):
                 results.append({'path': thpath, 'type': th})
                 break
@@ -529,13 +544,26 @@ def parse_yaml_linenumbers(data, filename):
         return node
 
     def construct_mapping(node, deep=False):
-        mapping = Constructor.construct_mapping(loader, node, deep=deep)
-        mapping[LINE_NUMBER_KEY] = node.__line__
+        if ANSIBLE_VERSION < 2:
+            mapping = Constructor.construct_mapping(loader, node, deep=deep)
+        else:
+            mapping = AnsibleConstructor.construct_mapping(loader, node, deep=deep)
+        if hasattr(node, '__line__'):
+            mapping[LINE_NUMBER_KEY] = node.__line__
+        else:
+            mapping[LINE_NUMBER_KEY] = mapping._line_number
         mapping[FILENAME_KEY] = filename
         return mapping
 
     try:
-        loader = yaml.Loader(data)
+        if ANSIBLE_VERSION < 2:
+            loader = yaml.Loader(data)
+        else:
+            import inspect
+            kwargs = {}
+            if 'vault_password' in inspect.getargspec(AnsibleLoader.__init__).args:
+                kwargs['vault_password'] = DEFAULT_VAULT_PASSWORD
+            loader = AnsibleLoader(data, **kwargs)
         loader.compose_node = compose_node
         loader.construct_mapping = construct_mapping
         data = loader.get_single_data()
